/* 
 * Mach Operating System
 * Copyright (c) 1991,1990 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS 
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they make and grant Carnegie the
 * rights to redistribute these changes.
 */

#ifndef	_ERROR_H
#define	_ERROR_H

#include <sys/errno.h>
extern int errno;

#if	defined(__GNUC__)
extern void fatal(const char *format, ...)
#if	__GNUC__ > 1
			__attribute__ ((format (printf, 1, 2),
					noreturn))
#endif
							;
extern void warn(const char *format, ...)
#if	__GNUC__ > 1
			__attribute__ ((format (printf, 1, 2)))
#endif
							;
extern void error(const char *format, ...)
#if	__GNUC__ > 1
			__attribute__ ((format (printf, 1, 2)))
#endif
							;
#else	/* not defined(__GNUC__) */
extern void fatal(const char *format, ...);
extern void warn(const char *format, ...);
extern void error(const char *format, ...);
#endif	/* defined(__GNUC__) */

extern const char *unix_error_string(int error_num);

extern int errors;
extern void set_program_name(const char *name);

#endif	/* _ERROR_H */
